@@ -7,6 +7,7 @@ import sys
 import queue
 import typing
 import uuid
+import weakref
 
 import mitmproxy.certs
 import mitmproxy.ctx
@@ -70,7 +71,7 @@ class SLTlsConfig(mitmproxy.addons.tlsconfig.TlsConfig):
         )
         self.certstore.certs = old_cert_store.certs
 
-    def tls_start_server(self, tls_start: tls.TlsStartData):
+    def tls_start_server(self, tls_start: tls.TlsData):
         super().tls_start_server(tls_start)
         # Since 2000 the recommendation per RFCs has been to only check SANs and not the CN field.
         # Most browsers do this, as does mitmproxy. The viewer does not, and the sim certs have no SAN
@@ -99,7 +100,7 @@ class IPCInterceptionAddon:
     """
     def __init__(self, flow_context: HTTPFlowContext):
         self.mitmproxy_ready = flow_context.mitmproxy_ready
-        self.intercepted_flows: typing.Dict[str, HTTPFlow] = {}
+        self.flows: weakref.WeakValueDictionary[str, HTTPFlow] = weakref.WeakValueDictionary()
         self.from_proxy_queue: multiprocessing.Queue = flow_context.from_proxy_queue
         self.to_proxy_queue: multiprocessing.Queue = flow_context.to_proxy_queue
         self.shutdown_signal: multiprocessing.Event = flow_context.shutdown_signal
@@ -134,8 +135,13 @@ class IPCInterceptionAddon:
                     await asyncio.sleep(0.001)
                     continue
                 if event_type == "callback":
-                    orig_flow = self.intercepted_flows.pop(flow_id)
+                    orig_flow = self.flows[flow_id]
                     orig_flow.set_state(flow_state)
+                elif event_type == "preempt":
+                    orig_flow = self.flows.get(flow_id)
+                    if orig_flow:
+                        orig_flow.intercept()
+                        orig_flow.set_state(flow_state)
                 elif event_type == "replay":
                     flow: HTTPFlow = HTTPFlow.from_state(flow_state)
                     # mitmproxy won't replay intercepted flows, this is an old flow so
@@ -157,8 +163,8 @@ class IPCInterceptionAddon:
         from_browser = "Mozilla" in flow.request.headers.get("User-Agent", "")
         flow.metadata["from_browser"] = from_browser
         # Only trust the "injected" header if not from a browser
-        was_injected = flow.request.headers.pop("X-Hippo-Injected", False)
-        if was_injected and not from_browser:
+        was_injected = flow.request.headers.pop("X-Hippo-Injected", "")
+        if was_injected == "1" and not from_browser:
             flow.metadata["request_injected"] = True
 
         # Does this request need the stupid hack around aiohttp's windows proactor bug
@@ -169,13 +175,13 @@ class IPCInterceptionAddon:
 
     def _queue_flow_interception(self, event_type: str, flow: HTTPFlow):
         flow.intercept()
-        self.intercepted_flows[flow.id] = flow
+        self.flows[flow.id] = flow
         self.from_proxy_queue.put((event_type, flow.get_state()), True)
 
     def responseheaders(self, flow: HTTPFlow):
         # The response was injected earlier in an earlier handler,
         # we don't want to touch this anymore.
-        if flow.metadata["response_injected"]:
+        if flow.metadata.get("response_injected"):
             return
 
         # Someone fucked up and put a mimetype in Content-Encoding.
@@ -187,7 +193,7 @@ class IPCInterceptionAddon:
 
     def response(self, flow: HTTPFlow):
         cap_data: typing.Optional[SerializedCapData] = flow.metadata.get("cap_data")
-        if flow.metadata["response_injected"] and cap_data and cap_data.asset_server_cap:
+        if flow.metadata.get("response_injected") and cap_data and cap_data.asset_server_cap:
             # Don't bother intercepting asset server requests where we injected a response.
             # We don't want to log them and they don't need any more processing by user hooks.
             return
@@ -197,10 +203,10 @@ class IPCInterceptionAddon:
 class SLMITMAddon(IPCInterceptionAddon):
     def responseheaders(self, flow: HTTPFlow):
         super().responseheaders(flow)
-        cap_data: typing.Optional[SerializedCapData] = flow.metadata["cap_data_ser"]
+        cap_data: typing.Optional[SerializedCapData] = flow.metadata.get("cap_data_ser")
 
         # Request came from the proxy itself, don't touch it.
-        if flow.metadata["request_injected"]:
+        if flow.metadata.get("request_injected"):
             return
 
         # This is an asset server response that we're not interested in intercepting.
@@ -209,7 +215,7 @@ class SLMITMAddon(IPCInterceptionAddon):
             # Can't stream if we injected our own response or we were asked not to stream
             if not flow.metadata["response_injected"] and flow.metadata["can_stream"]:
                 flow.response.stream = True
-        elif not cap_data and not flow.metadata["from_browser"]:
+        elif not cap_data and not flow.metadata.get("from_browser"):
             object_name = flow.response.headers.get("X-SecondLife-Object-Name", "")
             # Meh. Add some fake Cap data for this so it can be matched on.
             if object_name.startswith("#Firestorm LSL Bridge"):
@@ -229,10 +235,6 @@ class SLMITMMaster(mitmproxy.master.Master):
             SLMITMAddon(flow_context),
         )
 
-    def start_server(self):
-        self.start()
-        asyncio.ensure_future(self.running())
-
 
 def create_proxy_master(host, port, flow_context: HTTPFlowContext):  # pragma: no cover
     opts = mitmproxy.options.Options()
