@@ -401,7 +401,7 @@ class AbstractMessageLogEntry(abc.ABC):
         beautified = minidom.parseString(content).toprettyxml(indent="  ")
         # kill blank lines. will break cdata sections. meh.
         beautified = re.sub(r'\n\s*\n', '\n', beautified, flags=re.MULTILINE)
-        return re.sub(r'<([\w]+)>\s*</\1>', r'<\1></\1>',
+        return re.sub(r'<(\w+)>\s*</\1>', r'<\1></\1>',
                       beautified, flags=re.MULTILINE)
 
 
@@ -522,7 +522,7 @@ class HTTPMessageLogEntry(AbstractMessageLogEntry):
         buf.write(bytes(headers).decode("utf8", errors="replace"))
         buf.write("\r\n")
 
-        buf.write(message_body)
+        buf.write(message_body or "")
         return buf.getvalue()
 
     def request(self, beautify=False, replacements=None):
@@ -549,6 +549,12 @@ class HTTPMessageLogEntry(AbstractMessageLogEntry):
         return self._summary
 
     def _guess_content_type(self, message):
+        # SL's login service lies and says that its XML-RPC response is LLSD+XML.
+        # It is not, and it blows up the parser. It's been broken ever since the
+        # login rewrite and a fix is likely not forthcoming. I'm sick of seeing
+        # the traceback, so just hack around it.
+        if self.name == "LoginRequest":
+            return "application/xml"
         content_type = message.headers.get("Content-Type", "")
         if not message.content or content_type.startswith("application/llsd"):
             return content_type
